﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Windows.Devices.Bluetooth;
using Windows.Devices.Bluetooth.Advertisement;
using Windows.Devices.Bluetooth.GenericAttributeProfile;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace GattWriteIndicate
{
    public partial class Form1 : Form
    {
        BluetoothLEAdvertisementWatcher advWatcher;
        Guid serviceGuid = new Guid("569a1902-b87f-490c-92cb-11ba5ea5167c");
        Guid charRead = new Guid("569a2030-b87f-490c-92cb-11ba5ea5167c");
        Guid charWrite = new Guid("569a2031-b87f-490c-92cb-11ba5ea5167c");
        GattCharacteristic charLED, charBTN;

        bool readyService = false;//サービスが使用可能になるとtrue


        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            advWatcher = new BluetoothLEAdvertisementWatcher();
            advWatcher.SignalStrengthFilter.SamplingInterval = TimeSpan.FromMilliseconds(10000);
            advWatcher.ScanningMode = BluetoothLEScanningMode.Passive;
            advWatcher.Received += WatcherReceived;

            //ウォッチャースタート
            advWatcher.Start();
        }

        private void WatcherReceived(BluetoothLEAdvertisementWatcher sender, BluetoothLEAdvertisementReceivedEventArgs args)
        {
            CheckArgs(args);
        }

        private async void CheckArgs(BluetoothLEAdvertisementReceivedEventArgs args)
        {
            bool find = false;

            var bleServiceUUIDs = args.Advertisement.ServiceUuids;
            foreach (var uuidone in bleServiceUUIDs)
            {

                //serviceGuid 端末のGuid
                if (uuidone == serviceGuid)
                {
                    // 発見
                    find = true;
                    break;
                }
            }


            if (find)
            {
                try
                {
                    // スキャン停止
                    advWatcher.Stop();

                    //接続
                    var device = await BluetoothLEDevice.FromBluetoothAddressAsync(args.BluetoothAddress);

                    //サービスUUIDを使って目的のサービスを取得
                    var gattService = await device.GetGattServicesForUuidAsync(serviceGuid);

                    //LED(data_write)キャラクタリスティックUUIDを使って目的のキャラクタリスティックを取得
                    var char1 = await gattService.Services[0].GetCharacteristicsForUuidAsync(charWrite);

                    if (char1.Status == GattCommunicationStatus.Success)
                    {
                        charLED = char1.Characteristics.First();
                        if (charLED == null)
                        {
                            throw new Exception("データ書き込みはできません！");
                        }
                        //書き込み可能ならば。データを書き込む
                        if (charLED.CharacteristicProperties.HasFlag(GattCharacteristicProperties.Write))
                        {
                            //   Console.WriteLine("Start LED");
                        }
                    }

                    //BTN(data_read)キャラクタリスティックUUIDを使って目的のキャラクタリスティックを取得
                    var char2 = await gattService.Services[0].GetCharacteristicsForUuidAsync(charRead);

                    if (char2.Status == GattCommunicationStatus.Success)
                    {
                        charBTN = char2.Characteristics.First();
                        if (charBTN == null)
                        {
                            throw new Exception("データ読み込みはできません！");
                        }
                        //Indicate可能ならばコールバックを設定する
                        if (charBTN.CharacteristicProperties.HasFlag(GattCharacteristicProperties.Indicate))
                        {
                            await charBTN.WriteClientCharacteristicConfigurationDescriptorAsync(GattClientCharacteristicConfigurationDescriptorValue.Indicate);
                            charBTN.ValueChanged += characteristicChanged_BTN;
                            //   Console.WriteLine("Start BTN");
                            readyService = true;
                        }
                    }

                }

                catch (Exception ex)
                {
                    MessageBox.Show("例外", $"Exception...{ex.Message})", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    //Console.WriteLine($"Exception...{ex.Message})");
                }
            }
        }

        //BTN(data_read)用コールバック関数
        private void characteristicChanged_BTN(GattCharacteristic sender, GattValueChangedEventArgs eventArgs)
        {
            byte[] data = new byte[eventArgs.CharacteristicValue.Length];
            Windows.Storage.Streams.DataReader.FromBuffer(eventArgs.CharacteristicValue).ReadBytes(data);

            string text = System.Text.Encoding.ASCII.GetString(data);
            InputBox(text);
            return;
        }

        public void writeToBox(string msg)
        {
            inBox.Text = msg;
        }

        private void InputBox(string str)
        {
            Invoke(new Action<string>(writeToBox), str);
        }

        private void send_Click(object sender, EventArgs e)
        {
            while (readyService == false) ;

            string s = outBox.Text;
            byte[] OutData = System.Text.Encoding.ASCII.GetBytes(s);
            var result = charLED.WriteValueAsync(OutData.AsBuffer());
        }
    }

}
